/*
Copyright 2022 SUSE LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v1beta1

import (
	"context"
	"time"

	v1beta1 "github.com/rancher/elemental-operator/pkg/apis/elemental.cattle.io/v1beta1"
	scheme "github.com/rancher/elemental-operator/pkg/generated/clientset/versioned/scheme"
	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// MachineInventorySelectorsGetter has a method to return a MachineInventorySelectorInterface.
// A group's client should implement this interface.
type MachineInventorySelectorsGetter interface {
	MachineInventorySelectors(namespace string) MachineInventorySelectorInterface
}

// MachineInventorySelectorInterface has methods to work with MachineInventorySelector resources.
type MachineInventorySelectorInterface interface {
	Create(ctx context.Context, machineInventorySelector *v1beta1.MachineInventorySelector, opts v1.CreateOptions) (*v1beta1.MachineInventorySelector, error)
	Update(ctx context.Context, machineInventorySelector *v1beta1.MachineInventorySelector, opts v1.UpdateOptions) (*v1beta1.MachineInventorySelector, error)
	UpdateStatus(ctx context.Context, machineInventorySelector *v1beta1.MachineInventorySelector, opts v1.UpdateOptions) (*v1beta1.MachineInventorySelector, error)
	Delete(ctx context.Context, name string, opts v1.DeleteOptions) error
	DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error
	Get(ctx context.Context, name string, opts v1.GetOptions) (*v1beta1.MachineInventorySelector, error)
	List(ctx context.Context, opts v1.ListOptions) (*v1beta1.MachineInventorySelectorList, error)
	Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error)
	Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1beta1.MachineInventorySelector, err error)
	MachineInventorySelectorExpansion
}

// machineInventorySelectors implements MachineInventorySelectorInterface
type machineInventorySelectors struct {
	client rest.Interface
	ns     string
}

// newMachineInventorySelectors returns a MachineInventorySelectors
func newMachineInventorySelectors(c *ElementalV1beta1Client, namespace string) *machineInventorySelectors {
	return &machineInventorySelectors{
		client: c.RESTClient(),
		ns:     namespace,
	}
}

// Get takes name of the machineInventorySelector, and returns the corresponding machineInventorySelector object, and an error if there is any.
func (c *machineInventorySelectors) Get(ctx context.Context, name string, options v1.GetOptions) (result *v1beta1.MachineInventorySelector, err error) {
	result = &v1beta1.MachineInventorySelector{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("machineinventoryselectors").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do(ctx).
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of MachineInventorySelectors that match those selectors.
func (c *machineInventorySelectors) List(ctx context.Context, opts v1.ListOptions) (result *v1beta1.MachineInventorySelectorList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1beta1.MachineInventorySelectorList{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("machineinventoryselectors").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do(ctx).
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested machineInventorySelectors.
func (c *machineInventorySelectors) Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Namespace(c.ns).
		Resource("machineinventoryselectors").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch(ctx)
}

// Create takes the representation of a machineInventorySelector and creates it.  Returns the server's representation of the machineInventorySelector, and an error, if there is any.
func (c *machineInventorySelectors) Create(ctx context.Context, machineInventorySelector *v1beta1.MachineInventorySelector, opts v1.CreateOptions) (result *v1beta1.MachineInventorySelector, err error) {
	result = &v1beta1.MachineInventorySelector{}
	err = c.client.Post().
		Namespace(c.ns).
		Resource("machineinventoryselectors").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(machineInventorySelector).
		Do(ctx).
		Into(result)
	return
}

// Update takes the representation of a machineInventorySelector and updates it. Returns the server's representation of the machineInventorySelector, and an error, if there is any.
func (c *machineInventorySelectors) Update(ctx context.Context, machineInventorySelector *v1beta1.MachineInventorySelector, opts v1.UpdateOptions) (result *v1beta1.MachineInventorySelector, err error) {
	result = &v1beta1.MachineInventorySelector{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("machineinventoryselectors").
		Name(machineInventorySelector.Name).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(machineInventorySelector).
		Do(ctx).
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().
func (c *machineInventorySelectors) UpdateStatus(ctx context.Context, machineInventorySelector *v1beta1.MachineInventorySelector, opts v1.UpdateOptions) (result *v1beta1.MachineInventorySelector, err error) {
	result = &v1beta1.MachineInventorySelector{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("machineinventoryselectors").
		Name(machineInventorySelector.Name).
		SubResource("status").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(machineInventorySelector).
		Do(ctx).
		Into(result)
	return
}

// Delete takes name of the machineInventorySelector and deletes it. Returns an error if one occurs.
func (c *machineInventorySelectors) Delete(ctx context.Context, name string, opts v1.DeleteOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("machineinventoryselectors").
		Name(name).
		Body(&opts).
		Do(ctx).
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *machineInventorySelectors) DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error {
	var timeout time.Duration
	if listOpts.TimeoutSeconds != nil {
		timeout = time.Duration(*listOpts.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Namespace(c.ns).
		Resource("machineinventoryselectors").
		VersionedParams(&listOpts, scheme.ParameterCodec).
		Timeout(timeout).
		Body(&opts).
		Do(ctx).
		Error()
}

// Patch applies the patch and returns the patched machineInventorySelector.
func (c *machineInventorySelectors) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1beta1.MachineInventorySelector, err error) {
	result = &v1beta1.MachineInventorySelector{}
	err = c.client.Patch(pt).
		Namespace(c.ns).
		Resource("machineinventoryselectors").
		Name(name).
		SubResource(subresources...).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}
