/*
Copyright 2022 SUSE LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v1beta1

import (
	"context"
	"time"

	v1beta1 "github.com/rancher/elemental-operator/pkg/apis/elemental.cattle.io/v1beta1"
	"github.com/rancher/lasso/pkg/client"
	"github.com/rancher/lasso/pkg/controller"
	"github.com/rancher/wrangler/pkg/apply"
	"github.com/rancher/wrangler/pkg/condition"
	"github.com/rancher/wrangler/pkg/generic"
	"github.com/rancher/wrangler/pkg/kv"
	"k8s.io/apimachinery/pkg/api/equality"
	"k8s.io/apimachinery/pkg/api/errors"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/labels"
	"k8s.io/apimachinery/pkg/runtime"
	"k8s.io/apimachinery/pkg/runtime/schema"
	"k8s.io/apimachinery/pkg/types"
	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
	"k8s.io/apimachinery/pkg/watch"
	"k8s.io/client-go/tools/cache"
)

type MachineInventorySelectorHandler func(string, *v1beta1.MachineInventorySelector) (*v1beta1.MachineInventorySelector, error)

type MachineInventorySelectorController interface {
	generic.ControllerMeta
	MachineInventorySelectorClient

	OnChange(ctx context.Context, name string, sync MachineInventorySelectorHandler)
	OnRemove(ctx context.Context, name string, sync MachineInventorySelectorHandler)
	Enqueue(namespace, name string)
	EnqueueAfter(namespace, name string, duration time.Duration)

	Cache() MachineInventorySelectorCache
}

type MachineInventorySelectorClient interface {
	Create(*v1beta1.MachineInventorySelector) (*v1beta1.MachineInventorySelector, error)
	Update(*v1beta1.MachineInventorySelector) (*v1beta1.MachineInventorySelector, error)
	UpdateStatus(*v1beta1.MachineInventorySelector) (*v1beta1.MachineInventorySelector, error)
	Delete(namespace, name string, options *metav1.DeleteOptions) error
	Get(namespace, name string, options metav1.GetOptions) (*v1beta1.MachineInventorySelector, error)
	List(namespace string, opts metav1.ListOptions) (*v1beta1.MachineInventorySelectorList, error)
	Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error)
	Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (result *v1beta1.MachineInventorySelector, err error)
}

type MachineInventorySelectorCache interface {
	Get(namespace, name string) (*v1beta1.MachineInventorySelector, error)
	List(namespace string, selector labels.Selector) ([]*v1beta1.MachineInventorySelector, error)

	AddIndexer(indexName string, indexer MachineInventorySelectorIndexer)
	GetByIndex(indexName, key string) ([]*v1beta1.MachineInventorySelector, error)
}

type MachineInventorySelectorIndexer func(obj *v1beta1.MachineInventorySelector) ([]string, error)

type machineInventorySelectorController struct {
	controller    controller.SharedController
	client        *client.Client
	gvk           schema.GroupVersionKind
	groupResource schema.GroupResource
}

func NewMachineInventorySelectorController(gvk schema.GroupVersionKind, resource string, namespaced bool, controller controller.SharedControllerFactory) MachineInventorySelectorController {
	c := controller.ForResourceKind(gvk.GroupVersion().WithResource(resource), gvk.Kind, namespaced)
	return &machineInventorySelectorController{
		controller: c,
		client:     c.Client(),
		gvk:        gvk,
		groupResource: schema.GroupResource{
			Group:    gvk.Group,
			Resource: resource,
		},
	}
}

func FromMachineInventorySelectorHandlerToHandler(sync MachineInventorySelectorHandler) generic.Handler {
	return func(key string, obj runtime.Object) (ret runtime.Object, err error) {
		var v *v1beta1.MachineInventorySelector
		if obj == nil {
			v, err = sync(key, nil)
		} else {
			v, err = sync(key, obj.(*v1beta1.MachineInventorySelector))
		}
		if v == nil {
			return nil, err
		}
		return v, err
	}
}

func (c *machineInventorySelectorController) Updater() generic.Updater {
	return func(obj runtime.Object) (runtime.Object, error) {
		newObj, err := c.Update(obj.(*v1beta1.MachineInventorySelector))
		if newObj == nil {
			return nil, err
		}
		return newObj, err
	}
}

func UpdateMachineInventorySelectorDeepCopyOnChange(client MachineInventorySelectorClient, obj *v1beta1.MachineInventorySelector, handler func(obj *v1beta1.MachineInventorySelector) (*v1beta1.MachineInventorySelector, error)) (*v1beta1.MachineInventorySelector, error) {
	if obj == nil {
		return obj, nil
	}

	copyObj := obj.DeepCopy()
	newObj, err := handler(copyObj)
	if newObj != nil {
		copyObj = newObj
	}
	if obj.ResourceVersion == copyObj.ResourceVersion && !equality.Semantic.DeepEqual(obj, copyObj) {
		return client.Update(copyObj)
	}

	return copyObj, err
}

func (c *machineInventorySelectorController) AddGenericHandler(ctx context.Context, name string, handler generic.Handler) {
	c.controller.RegisterHandler(ctx, name, controller.SharedControllerHandlerFunc(handler))
}

func (c *machineInventorySelectorController) AddGenericRemoveHandler(ctx context.Context, name string, handler generic.Handler) {
	c.AddGenericHandler(ctx, name, generic.NewRemoveHandler(name, c.Updater(), handler))
}

func (c *machineInventorySelectorController) OnChange(ctx context.Context, name string, sync MachineInventorySelectorHandler) {
	c.AddGenericHandler(ctx, name, FromMachineInventorySelectorHandlerToHandler(sync))
}

func (c *machineInventorySelectorController) OnRemove(ctx context.Context, name string, sync MachineInventorySelectorHandler) {
	c.AddGenericHandler(ctx, name, generic.NewRemoveHandler(name, c.Updater(), FromMachineInventorySelectorHandlerToHandler(sync)))
}

func (c *machineInventorySelectorController) Enqueue(namespace, name string) {
	c.controller.Enqueue(namespace, name)
}

func (c *machineInventorySelectorController) EnqueueAfter(namespace, name string, duration time.Duration) {
	c.controller.EnqueueAfter(namespace, name, duration)
}

func (c *machineInventorySelectorController) Informer() cache.SharedIndexInformer {
	return c.controller.Informer()
}

func (c *machineInventorySelectorController) GroupVersionKind() schema.GroupVersionKind {
	return c.gvk
}

func (c *machineInventorySelectorController) Cache() MachineInventorySelectorCache {
	return &machineInventorySelectorCache{
		indexer:  c.Informer().GetIndexer(),
		resource: c.groupResource,
	}
}

func (c *machineInventorySelectorController) Create(obj *v1beta1.MachineInventorySelector) (*v1beta1.MachineInventorySelector, error) {
	result := &v1beta1.MachineInventorySelector{}
	return result, c.client.Create(context.TODO(), obj.Namespace, obj, result, metav1.CreateOptions{})
}

func (c *machineInventorySelectorController) Update(obj *v1beta1.MachineInventorySelector) (*v1beta1.MachineInventorySelector, error) {
	result := &v1beta1.MachineInventorySelector{}
	return result, c.client.Update(context.TODO(), obj.Namespace, obj, result, metav1.UpdateOptions{})
}

func (c *machineInventorySelectorController) UpdateStatus(obj *v1beta1.MachineInventorySelector) (*v1beta1.MachineInventorySelector, error) {
	result := &v1beta1.MachineInventorySelector{}
	return result, c.client.UpdateStatus(context.TODO(), obj.Namespace, obj, result, metav1.UpdateOptions{})
}

func (c *machineInventorySelectorController) Delete(namespace, name string, options *metav1.DeleteOptions) error {
	if options == nil {
		options = &metav1.DeleteOptions{}
	}
	return c.client.Delete(context.TODO(), namespace, name, *options)
}

func (c *machineInventorySelectorController) Get(namespace, name string, options metav1.GetOptions) (*v1beta1.MachineInventorySelector, error) {
	result := &v1beta1.MachineInventorySelector{}
	return result, c.client.Get(context.TODO(), namespace, name, result, options)
}

func (c *machineInventorySelectorController) List(namespace string, opts metav1.ListOptions) (*v1beta1.MachineInventorySelectorList, error) {
	result := &v1beta1.MachineInventorySelectorList{}
	return result, c.client.List(context.TODO(), namespace, result, opts)
}

func (c *machineInventorySelectorController) Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error) {
	return c.client.Watch(context.TODO(), namespace, opts)
}

func (c *machineInventorySelectorController) Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (*v1beta1.MachineInventorySelector, error) {
	result := &v1beta1.MachineInventorySelector{}
	return result, c.client.Patch(context.TODO(), namespace, name, pt, data, result, metav1.PatchOptions{}, subresources...)
}

type machineInventorySelectorCache struct {
	indexer  cache.Indexer
	resource schema.GroupResource
}

func (c *machineInventorySelectorCache) Get(namespace, name string) (*v1beta1.MachineInventorySelector, error) {
	obj, exists, err := c.indexer.GetByKey(namespace + "/" + name)
	if err != nil {
		return nil, err
	}
	if !exists {
		return nil, errors.NewNotFound(c.resource, name)
	}
	return obj.(*v1beta1.MachineInventorySelector), nil
}

func (c *machineInventorySelectorCache) List(namespace string, selector labels.Selector) (ret []*v1beta1.MachineInventorySelector, err error) {

	err = cache.ListAllByNamespace(c.indexer, namespace, selector, func(m interface{}) {
		ret = append(ret, m.(*v1beta1.MachineInventorySelector))
	})

	return ret, err
}

func (c *machineInventorySelectorCache) AddIndexer(indexName string, indexer MachineInventorySelectorIndexer) {
	utilruntime.Must(c.indexer.AddIndexers(map[string]cache.IndexFunc{
		indexName: func(obj interface{}) (strings []string, e error) {
			return indexer(obj.(*v1beta1.MachineInventorySelector))
		},
	}))
}

func (c *machineInventorySelectorCache) GetByIndex(indexName, key string) (result []*v1beta1.MachineInventorySelector, err error) {
	objs, err := c.indexer.ByIndex(indexName, key)
	if err != nil {
		return nil, err
	}
	result = make([]*v1beta1.MachineInventorySelector, 0, len(objs))
	for _, obj := range objs {
		result = append(result, obj.(*v1beta1.MachineInventorySelector))
	}
	return result, nil
}

type MachineInventorySelectorStatusHandler func(obj *v1beta1.MachineInventorySelector, status v1beta1.MachineInventorySelectorStatus) (v1beta1.MachineInventorySelectorStatus, error)

type MachineInventorySelectorGeneratingHandler func(obj *v1beta1.MachineInventorySelector, status v1beta1.MachineInventorySelectorStatus) ([]runtime.Object, v1beta1.MachineInventorySelectorStatus, error)

func RegisterMachineInventorySelectorStatusHandler(ctx context.Context, controller MachineInventorySelectorController, condition condition.Cond, name string, handler MachineInventorySelectorStatusHandler) {
	statusHandler := &machineInventorySelectorStatusHandler{
		client:    controller,
		condition: condition,
		handler:   handler,
	}
	controller.AddGenericHandler(ctx, name, FromMachineInventorySelectorHandlerToHandler(statusHandler.sync))
}

func RegisterMachineInventorySelectorGeneratingHandler(ctx context.Context, controller MachineInventorySelectorController, apply apply.Apply,
	condition condition.Cond, name string, handler MachineInventorySelectorGeneratingHandler, opts *generic.GeneratingHandlerOptions) {
	statusHandler := &machineInventorySelectorGeneratingHandler{
		MachineInventorySelectorGeneratingHandler: handler,
		apply: apply,
		name:  name,
		gvk:   controller.GroupVersionKind(),
	}
	if opts != nil {
		statusHandler.opts = *opts
	}
	controller.OnChange(ctx, name, statusHandler.Remove)
	RegisterMachineInventorySelectorStatusHandler(ctx, controller, condition, name, statusHandler.Handle)
}

type machineInventorySelectorStatusHandler struct {
	client    MachineInventorySelectorClient
	condition condition.Cond
	handler   MachineInventorySelectorStatusHandler
}

func (a *machineInventorySelectorStatusHandler) sync(key string, obj *v1beta1.MachineInventorySelector) (*v1beta1.MachineInventorySelector, error) {
	if obj == nil {
		return obj, nil
	}

	origStatus := obj.Status.DeepCopy()
	obj = obj.DeepCopy()
	newStatus, err := a.handler(obj, obj.Status)
	if err != nil {
		// Revert to old status on error
		newStatus = *origStatus.DeepCopy()
	}

	if a.condition != "" {
		if errors.IsConflict(err) {
			a.condition.SetError(&newStatus, "", nil)
		} else {
			a.condition.SetError(&newStatus, "", err)
		}
	}
	if !equality.Semantic.DeepEqual(origStatus, &newStatus) {
		if a.condition != "" {
			// Since status has changed, update the lastUpdatedTime
			a.condition.LastUpdated(&newStatus, time.Now().UTC().Format(time.RFC3339))
		}

		var newErr error
		obj.Status = newStatus
		newObj, newErr := a.client.UpdateStatus(obj)
		if err == nil {
			err = newErr
		}
		if newErr == nil {
			obj = newObj
		}
	}
	return obj, err
}

type machineInventorySelectorGeneratingHandler struct {
	MachineInventorySelectorGeneratingHandler
	apply apply.Apply
	opts  generic.GeneratingHandlerOptions
	gvk   schema.GroupVersionKind
	name  string
}

func (a *machineInventorySelectorGeneratingHandler) Remove(key string, obj *v1beta1.MachineInventorySelector) (*v1beta1.MachineInventorySelector, error) {
	if obj != nil {
		return obj, nil
	}

	obj = &v1beta1.MachineInventorySelector{}
	obj.Namespace, obj.Name = kv.RSplit(key, "/")
	obj.SetGroupVersionKind(a.gvk)

	return nil, generic.ConfigureApplyForObject(a.apply, obj, &a.opts).
		WithOwner(obj).
		WithSetID(a.name).
		ApplyObjects()
}

func (a *machineInventorySelectorGeneratingHandler) Handle(obj *v1beta1.MachineInventorySelector, status v1beta1.MachineInventorySelectorStatus) (v1beta1.MachineInventorySelectorStatus, error) {
	if !obj.DeletionTimestamp.IsZero() {
		return status, nil
	}

	objs, newStatus, err := a.MachineInventorySelectorGeneratingHandler(obj, status)
	if err != nil {
		return newStatus, err
	}

	return newStatus, generic.ConfigureApplyForObject(a.apply, obj, &a.opts).
		WithOwner(obj).
		WithSetID(a.name).
		ApplyObjects(objs...)
}
