/*
Copyright 2022 SUSE LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v1beta1

import (
	"context"
	"time"

	v1beta1 "github.com/rancher/elemental-operator/pkg/apis/elemental.cattle.io/v1beta1"
	"github.com/rancher/lasso/pkg/client"
	"github.com/rancher/lasso/pkg/controller"
	"github.com/rancher/wrangler/pkg/apply"
	"github.com/rancher/wrangler/pkg/condition"
	"github.com/rancher/wrangler/pkg/generic"
	"github.com/rancher/wrangler/pkg/kv"
	"k8s.io/apimachinery/pkg/api/equality"
	"k8s.io/apimachinery/pkg/api/errors"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/labels"
	"k8s.io/apimachinery/pkg/runtime"
	"k8s.io/apimachinery/pkg/runtime/schema"
	"k8s.io/apimachinery/pkg/types"
	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
	"k8s.io/apimachinery/pkg/watch"
	"k8s.io/client-go/tools/cache"
)

type MachineRegistrationHandler func(string, *v1beta1.MachineRegistration) (*v1beta1.MachineRegistration, error)

type MachineRegistrationController interface {
	generic.ControllerMeta
	MachineRegistrationClient

	OnChange(ctx context.Context, name string, sync MachineRegistrationHandler)
	OnRemove(ctx context.Context, name string, sync MachineRegistrationHandler)
	Enqueue(namespace, name string)
	EnqueueAfter(namespace, name string, duration time.Duration)

	Cache() MachineRegistrationCache
}

type MachineRegistrationClient interface {
	Create(*v1beta1.MachineRegistration) (*v1beta1.MachineRegistration, error)
	Update(*v1beta1.MachineRegistration) (*v1beta1.MachineRegistration, error)
	UpdateStatus(*v1beta1.MachineRegistration) (*v1beta1.MachineRegistration, error)
	Delete(namespace, name string, options *metav1.DeleteOptions) error
	Get(namespace, name string, options metav1.GetOptions) (*v1beta1.MachineRegistration, error)
	List(namespace string, opts metav1.ListOptions) (*v1beta1.MachineRegistrationList, error)
	Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error)
	Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (result *v1beta1.MachineRegistration, err error)
}

type MachineRegistrationCache interface {
	Get(namespace, name string) (*v1beta1.MachineRegistration, error)
	List(namespace string, selector labels.Selector) ([]*v1beta1.MachineRegistration, error)

	AddIndexer(indexName string, indexer MachineRegistrationIndexer)
	GetByIndex(indexName, key string) ([]*v1beta1.MachineRegistration, error)
}

type MachineRegistrationIndexer func(obj *v1beta1.MachineRegistration) ([]string, error)

type machineRegistrationController struct {
	controller    controller.SharedController
	client        *client.Client
	gvk           schema.GroupVersionKind
	groupResource schema.GroupResource
}

func NewMachineRegistrationController(gvk schema.GroupVersionKind, resource string, namespaced bool, controller controller.SharedControllerFactory) MachineRegistrationController {
	c := controller.ForResourceKind(gvk.GroupVersion().WithResource(resource), gvk.Kind, namespaced)
	return &machineRegistrationController{
		controller: c,
		client:     c.Client(),
		gvk:        gvk,
		groupResource: schema.GroupResource{
			Group:    gvk.Group,
			Resource: resource,
		},
	}
}

func FromMachineRegistrationHandlerToHandler(sync MachineRegistrationHandler) generic.Handler {
	return func(key string, obj runtime.Object) (ret runtime.Object, err error) {
		var v *v1beta1.MachineRegistration
		if obj == nil {
			v, err = sync(key, nil)
		} else {
			v, err = sync(key, obj.(*v1beta1.MachineRegistration))
		}
		if v == nil {
			return nil, err
		}
		return v, err
	}
}

func (c *machineRegistrationController) Updater() generic.Updater {
	return func(obj runtime.Object) (runtime.Object, error) {
		newObj, err := c.Update(obj.(*v1beta1.MachineRegistration))
		if newObj == nil {
			return nil, err
		}
		return newObj, err
	}
}

func UpdateMachineRegistrationDeepCopyOnChange(client MachineRegistrationClient, obj *v1beta1.MachineRegistration, handler func(obj *v1beta1.MachineRegistration) (*v1beta1.MachineRegistration, error)) (*v1beta1.MachineRegistration, error) {
	if obj == nil {
		return obj, nil
	}

	copyObj := obj.DeepCopy()
	newObj, err := handler(copyObj)
	if newObj != nil {
		copyObj = newObj
	}
	if obj.ResourceVersion == copyObj.ResourceVersion && !equality.Semantic.DeepEqual(obj, copyObj) {
		return client.Update(copyObj)
	}

	return copyObj, err
}

func (c *machineRegistrationController) AddGenericHandler(ctx context.Context, name string, handler generic.Handler) {
	c.controller.RegisterHandler(ctx, name, controller.SharedControllerHandlerFunc(handler))
}

func (c *machineRegistrationController) AddGenericRemoveHandler(ctx context.Context, name string, handler generic.Handler) {
	c.AddGenericHandler(ctx, name, generic.NewRemoveHandler(name, c.Updater(), handler))
}

func (c *machineRegistrationController) OnChange(ctx context.Context, name string, sync MachineRegistrationHandler) {
	c.AddGenericHandler(ctx, name, FromMachineRegistrationHandlerToHandler(sync))
}

func (c *machineRegistrationController) OnRemove(ctx context.Context, name string, sync MachineRegistrationHandler) {
	c.AddGenericHandler(ctx, name, generic.NewRemoveHandler(name, c.Updater(), FromMachineRegistrationHandlerToHandler(sync)))
}

func (c *machineRegistrationController) Enqueue(namespace, name string) {
	c.controller.Enqueue(namespace, name)
}

func (c *machineRegistrationController) EnqueueAfter(namespace, name string, duration time.Duration) {
	c.controller.EnqueueAfter(namespace, name, duration)
}

func (c *machineRegistrationController) Informer() cache.SharedIndexInformer {
	return c.controller.Informer()
}

func (c *machineRegistrationController) GroupVersionKind() schema.GroupVersionKind {
	return c.gvk
}

func (c *machineRegistrationController) Cache() MachineRegistrationCache {
	return &machineRegistrationCache{
		indexer:  c.Informer().GetIndexer(),
		resource: c.groupResource,
	}
}

func (c *machineRegistrationController) Create(obj *v1beta1.MachineRegistration) (*v1beta1.MachineRegistration, error) {
	result := &v1beta1.MachineRegistration{}
	return result, c.client.Create(context.TODO(), obj.Namespace, obj, result, metav1.CreateOptions{})
}

func (c *machineRegistrationController) Update(obj *v1beta1.MachineRegistration) (*v1beta1.MachineRegistration, error) {
	result := &v1beta1.MachineRegistration{}
	return result, c.client.Update(context.TODO(), obj.Namespace, obj, result, metav1.UpdateOptions{})
}

func (c *machineRegistrationController) UpdateStatus(obj *v1beta1.MachineRegistration) (*v1beta1.MachineRegistration, error) {
	result := &v1beta1.MachineRegistration{}
	return result, c.client.UpdateStatus(context.TODO(), obj.Namespace, obj, result, metav1.UpdateOptions{})
}

func (c *machineRegistrationController) Delete(namespace, name string, options *metav1.DeleteOptions) error {
	if options == nil {
		options = &metav1.DeleteOptions{}
	}
	return c.client.Delete(context.TODO(), namespace, name, *options)
}

func (c *machineRegistrationController) Get(namespace, name string, options metav1.GetOptions) (*v1beta1.MachineRegistration, error) {
	result := &v1beta1.MachineRegistration{}
	return result, c.client.Get(context.TODO(), namespace, name, result, options)
}

func (c *machineRegistrationController) List(namespace string, opts metav1.ListOptions) (*v1beta1.MachineRegistrationList, error) {
	result := &v1beta1.MachineRegistrationList{}
	return result, c.client.List(context.TODO(), namespace, result, opts)
}

func (c *machineRegistrationController) Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error) {
	return c.client.Watch(context.TODO(), namespace, opts)
}

func (c *machineRegistrationController) Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (*v1beta1.MachineRegistration, error) {
	result := &v1beta1.MachineRegistration{}
	return result, c.client.Patch(context.TODO(), namespace, name, pt, data, result, metav1.PatchOptions{}, subresources...)
}

type machineRegistrationCache struct {
	indexer  cache.Indexer
	resource schema.GroupResource
}

func (c *machineRegistrationCache) Get(namespace, name string) (*v1beta1.MachineRegistration, error) {
	obj, exists, err := c.indexer.GetByKey(namespace + "/" + name)
	if err != nil {
		return nil, err
	}
	if !exists {
		return nil, errors.NewNotFound(c.resource, name)
	}
	return obj.(*v1beta1.MachineRegistration), nil
}

func (c *machineRegistrationCache) List(namespace string, selector labels.Selector) (ret []*v1beta1.MachineRegistration, err error) {

	err = cache.ListAllByNamespace(c.indexer, namespace, selector, func(m interface{}) {
		ret = append(ret, m.(*v1beta1.MachineRegistration))
	})

	return ret, err
}

func (c *machineRegistrationCache) AddIndexer(indexName string, indexer MachineRegistrationIndexer) {
	utilruntime.Must(c.indexer.AddIndexers(map[string]cache.IndexFunc{
		indexName: func(obj interface{}) (strings []string, e error) {
			return indexer(obj.(*v1beta1.MachineRegistration))
		},
	}))
}

func (c *machineRegistrationCache) GetByIndex(indexName, key string) (result []*v1beta1.MachineRegistration, err error) {
	objs, err := c.indexer.ByIndex(indexName, key)
	if err != nil {
		return nil, err
	}
	result = make([]*v1beta1.MachineRegistration, 0, len(objs))
	for _, obj := range objs {
		result = append(result, obj.(*v1beta1.MachineRegistration))
	}
	return result, nil
}

type MachineRegistrationStatusHandler func(obj *v1beta1.MachineRegistration, status v1beta1.MachineRegistrationStatus) (v1beta1.MachineRegistrationStatus, error)

type MachineRegistrationGeneratingHandler func(obj *v1beta1.MachineRegistration, status v1beta1.MachineRegistrationStatus) ([]runtime.Object, v1beta1.MachineRegistrationStatus, error)

func RegisterMachineRegistrationStatusHandler(ctx context.Context, controller MachineRegistrationController, condition condition.Cond, name string, handler MachineRegistrationStatusHandler) {
	statusHandler := &machineRegistrationStatusHandler{
		client:    controller,
		condition: condition,
		handler:   handler,
	}
	controller.AddGenericHandler(ctx, name, FromMachineRegistrationHandlerToHandler(statusHandler.sync))
}

func RegisterMachineRegistrationGeneratingHandler(ctx context.Context, controller MachineRegistrationController, apply apply.Apply,
	condition condition.Cond, name string, handler MachineRegistrationGeneratingHandler, opts *generic.GeneratingHandlerOptions) {
	statusHandler := &machineRegistrationGeneratingHandler{
		MachineRegistrationGeneratingHandler: handler,
		apply:                                apply,
		name:                                 name,
		gvk:                                  controller.GroupVersionKind(),
	}
	if opts != nil {
		statusHandler.opts = *opts
	}
	controller.OnChange(ctx, name, statusHandler.Remove)
	RegisterMachineRegistrationStatusHandler(ctx, controller, condition, name, statusHandler.Handle)
}

type machineRegistrationStatusHandler struct {
	client    MachineRegistrationClient
	condition condition.Cond
	handler   MachineRegistrationStatusHandler
}

func (a *machineRegistrationStatusHandler) sync(key string, obj *v1beta1.MachineRegistration) (*v1beta1.MachineRegistration, error) {
	if obj == nil {
		return obj, nil
	}

	origStatus := obj.Status.DeepCopy()
	obj = obj.DeepCopy()
	newStatus, err := a.handler(obj, obj.Status)
	if err != nil {
		// Revert to old status on error
		newStatus = *origStatus.DeepCopy()
	}

	if a.condition != "" {
		if errors.IsConflict(err) {
			a.condition.SetError(&newStatus, "", nil)
		} else {
			a.condition.SetError(&newStatus, "", err)
		}
	}
	if !equality.Semantic.DeepEqual(origStatus, &newStatus) {
		if a.condition != "" {
			// Since status has changed, update the lastUpdatedTime
			a.condition.LastUpdated(&newStatus, time.Now().UTC().Format(time.RFC3339))
		}

		var newErr error
		obj.Status = newStatus
		newObj, newErr := a.client.UpdateStatus(obj)
		if err == nil {
			err = newErr
		}
		if newErr == nil {
			obj = newObj
		}
	}
	return obj, err
}

type machineRegistrationGeneratingHandler struct {
	MachineRegistrationGeneratingHandler
	apply apply.Apply
	opts  generic.GeneratingHandlerOptions
	gvk   schema.GroupVersionKind
	name  string
}

func (a *machineRegistrationGeneratingHandler) Remove(key string, obj *v1beta1.MachineRegistration) (*v1beta1.MachineRegistration, error) {
	if obj != nil {
		return obj, nil
	}

	obj = &v1beta1.MachineRegistration{}
	obj.Namespace, obj.Name = kv.RSplit(key, "/")
	obj.SetGroupVersionKind(a.gvk)

	return nil, generic.ConfigureApplyForObject(a.apply, obj, &a.opts).
		WithOwner(obj).
		WithSetID(a.name).
		ApplyObjects()
}

func (a *machineRegistrationGeneratingHandler) Handle(obj *v1beta1.MachineRegistration, status v1beta1.MachineRegistrationStatus) (v1beta1.MachineRegistrationStatus, error) {
	if !obj.DeletionTimestamp.IsZero() {
		return status, nil
	}

	objs, newStatus, err := a.MachineRegistrationGeneratingHandler(obj, status)
	if err != nil {
		return newStatus, err
	}

	return newStatus, generic.ConfigureApplyForObject(a.apply, obj, &a.opts).
		WithOwner(obj).
		WithSetID(a.name).
		ApplyObjects(objs...)
}
