/*
Copyright 2022 SUSE LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v1beta1

import (
	"context"
	"time"

	v1beta1 "github.com/rancher/elemental-operator/pkg/apis/elemental.cattle.io/v1beta1"
	"github.com/rancher/lasso/pkg/client"
	"github.com/rancher/lasso/pkg/controller"
	"github.com/rancher/wrangler/pkg/apply"
	"github.com/rancher/wrangler/pkg/condition"
	"github.com/rancher/wrangler/pkg/generic"
	"github.com/rancher/wrangler/pkg/kv"
	"k8s.io/apimachinery/pkg/api/equality"
	"k8s.io/apimachinery/pkg/api/errors"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/labels"
	"k8s.io/apimachinery/pkg/runtime"
	"k8s.io/apimachinery/pkg/runtime/schema"
	"k8s.io/apimachinery/pkg/types"
	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
	"k8s.io/apimachinery/pkg/watch"
	"k8s.io/client-go/tools/cache"
)

type ManagedOSVersionHandler func(string, *v1beta1.ManagedOSVersion) (*v1beta1.ManagedOSVersion, error)

type ManagedOSVersionController interface {
	generic.ControllerMeta
	ManagedOSVersionClient

	OnChange(ctx context.Context, name string, sync ManagedOSVersionHandler)
	OnRemove(ctx context.Context, name string, sync ManagedOSVersionHandler)
	Enqueue(namespace, name string)
	EnqueueAfter(namespace, name string, duration time.Duration)

	Cache() ManagedOSVersionCache
}

type ManagedOSVersionClient interface {
	Create(*v1beta1.ManagedOSVersion) (*v1beta1.ManagedOSVersion, error)
	Update(*v1beta1.ManagedOSVersion) (*v1beta1.ManagedOSVersion, error)
	UpdateStatus(*v1beta1.ManagedOSVersion) (*v1beta1.ManagedOSVersion, error)
	Delete(namespace, name string, options *metav1.DeleteOptions) error
	Get(namespace, name string, options metav1.GetOptions) (*v1beta1.ManagedOSVersion, error)
	List(namespace string, opts metav1.ListOptions) (*v1beta1.ManagedOSVersionList, error)
	Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error)
	Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (result *v1beta1.ManagedOSVersion, err error)
}

type ManagedOSVersionCache interface {
	Get(namespace, name string) (*v1beta1.ManagedOSVersion, error)
	List(namespace string, selector labels.Selector) ([]*v1beta1.ManagedOSVersion, error)

	AddIndexer(indexName string, indexer ManagedOSVersionIndexer)
	GetByIndex(indexName, key string) ([]*v1beta1.ManagedOSVersion, error)
}

type ManagedOSVersionIndexer func(obj *v1beta1.ManagedOSVersion) ([]string, error)

type managedOSVersionController struct {
	controller    controller.SharedController
	client        *client.Client
	gvk           schema.GroupVersionKind
	groupResource schema.GroupResource
}

func NewManagedOSVersionController(gvk schema.GroupVersionKind, resource string, namespaced bool, controller controller.SharedControllerFactory) ManagedOSVersionController {
	c := controller.ForResourceKind(gvk.GroupVersion().WithResource(resource), gvk.Kind, namespaced)
	return &managedOSVersionController{
		controller: c,
		client:     c.Client(),
		gvk:        gvk,
		groupResource: schema.GroupResource{
			Group:    gvk.Group,
			Resource: resource,
		},
	}
}

func FromManagedOSVersionHandlerToHandler(sync ManagedOSVersionHandler) generic.Handler {
	return func(key string, obj runtime.Object) (ret runtime.Object, err error) {
		var v *v1beta1.ManagedOSVersion
		if obj == nil {
			v, err = sync(key, nil)
		} else {
			v, err = sync(key, obj.(*v1beta1.ManagedOSVersion))
		}
		if v == nil {
			return nil, err
		}
		return v, err
	}
}

func (c *managedOSVersionController) Updater() generic.Updater {
	return func(obj runtime.Object) (runtime.Object, error) {
		newObj, err := c.Update(obj.(*v1beta1.ManagedOSVersion))
		if newObj == nil {
			return nil, err
		}
		return newObj, err
	}
}

func UpdateManagedOSVersionDeepCopyOnChange(client ManagedOSVersionClient, obj *v1beta1.ManagedOSVersion, handler func(obj *v1beta1.ManagedOSVersion) (*v1beta1.ManagedOSVersion, error)) (*v1beta1.ManagedOSVersion, error) {
	if obj == nil {
		return obj, nil
	}

	copyObj := obj.DeepCopy()
	newObj, err := handler(copyObj)
	if newObj != nil {
		copyObj = newObj
	}
	if obj.ResourceVersion == copyObj.ResourceVersion && !equality.Semantic.DeepEqual(obj, copyObj) {
		return client.Update(copyObj)
	}

	return copyObj, err
}

func (c *managedOSVersionController) AddGenericHandler(ctx context.Context, name string, handler generic.Handler) {
	c.controller.RegisterHandler(ctx, name, controller.SharedControllerHandlerFunc(handler))
}

func (c *managedOSVersionController) AddGenericRemoveHandler(ctx context.Context, name string, handler generic.Handler) {
	c.AddGenericHandler(ctx, name, generic.NewRemoveHandler(name, c.Updater(), handler))
}

func (c *managedOSVersionController) OnChange(ctx context.Context, name string, sync ManagedOSVersionHandler) {
	c.AddGenericHandler(ctx, name, FromManagedOSVersionHandlerToHandler(sync))
}

func (c *managedOSVersionController) OnRemove(ctx context.Context, name string, sync ManagedOSVersionHandler) {
	c.AddGenericHandler(ctx, name, generic.NewRemoveHandler(name, c.Updater(), FromManagedOSVersionHandlerToHandler(sync)))
}

func (c *managedOSVersionController) Enqueue(namespace, name string) {
	c.controller.Enqueue(namespace, name)
}

func (c *managedOSVersionController) EnqueueAfter(namespace, name string, duration time.Duration) {
	c.controller.EnqueueAfter(namespace, name, duration)
}

func (c *managedOSVersionController) Informer() cache.SharedIndexInformer {
	return c.controller.Informer()
}

func (c *managedOSVersionController) GroupVersionKind() schema.GroupVersionKind {
	return c.gvk
}

func (c *managedOSVersionController) Cache() ManagedOSVersionCache {
	return &managedOSVersionCache{
		indexer:  c.Informer().GetIndexer(),
		resource: c.groupResource,
	}
}

func (c *managedOSVersionController) Create(obj *v1beta1.ManagedOSVersion) (*v1beta1.ManagedOSVersion, error) {
	result := &v1beta1.ManagedOSVersion{}
	return result, c.client.Create(context.TODO(), obj.Namespace, obj, result, metav1.CreateOptions{})
}

func (c *managedOSVersionController) Update(obj *v1beta1.ManagedOSVersion) (*v1beta1.ManagedOSVersion, error) {
	result := &v1beta1.ManagedOSVersion{}
	return result, c.client.Update(context.TODO(), obj.Namespace, obj, result, metav1.UpdateOptions{})
}

func (c *managedOSVersionController) UpdateStatus(obj *v1beta1.ManagedOSVersion) (*v1beta1.ManagedOSVersion, error) {
	result := &v1beta1.ManagedOSVersion{}
	return result, c.client.UpdateStatus(context.TODO(), obj.Namespace, obj, result, metav1.UpdateOptions{})
}

func (c *managedOSVersionController) Delete(namespace, name string, options *metav1.DeleteOptions) error {
	if options == nil {
		options = &metav1.DeleteOptions{}
	}
	return c.client.Delete(context.TODO(), namespace, name, *options)
}

func (c *managedOSVersionController) Get(namespace, name string, options metav1.GetOptions) (*v1beta1.ManagedOSVersion, error) {
	result := &v1beta1.ManagedOSVersion{}
	return result, c.client.Get(context.TODO(), namespace, name, result, options)
}

func (c *managedOSVersionController) List(namespace string, opts metav1.ListOptions) (*v1beta1.ManagedOSVersionList, error) {
	result := &v1beta1.ManagedOSVersionList{}
	return result, c.client.List(context.TODO(), namespace, result, opts)
}

func (c *managedOSVersionController) Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error) {
	return c.client.Watch(context.TODO(), namespace, opts)
}

func (c *managedOSVersionController) Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (*v1beta1.ManagedOSVersion, error) {
	result := &v1beta1.ManagedOSVersion{}
	return result, c.client.Patch(context.TODO(), namespace, name, pt, data, result, metav1.PatchOptions{}, subresources...)
}

type managedOSVersionCache struct {
	indexer  cache.Indexer
	resource schema.GroupResource
}

func (c *managedOSVersionCache) Get(namespace, name string) (*v1beta1.ManagedOSVersion, error) {
	obj, exists, err := c.indexer.GetByKey(namespace + "/" + name)
	if err != nil {
		return nil, err
	}
	if !exists {
		return nil, errors.NewNotFound(c.resource, name)
	}
	return obj.(*v1beta1.ManagedOSVersion), nil
}

func (c *managedOSVersionCache) List(namespace string, selector labels.Selector) (ret []*v1beta1.ManagedOSVersion, err error) {

	err = cache.ListAllByNamespace(c.indexer, namespace, selector, func(m interface{}) {
		ret = append(ret, m.(*v1beta1.ManagedOSVersion))
	})

	return ret, err
}

func (c *managedOSVersionCache) AddIndexer(indexName string, indexer ManagedOSVersionIndexer) {
	utilruntime.Must(c.indexer.AddIndexers(map[string]cache.IndexFunc{
		indexName: func(obj interface{}) (strings []string, e error) {
			return indexer(obj.(*v1beta1.ManagedOSVersion))
		},
	}))
}

func (c *managedOSVersionCache) GetByIndex(indexName, key string) (result []*v1beta1.ManagedOSVersion, err error) {
	objs, err := c.indexer.ByIndex(indexName, key)
	if err != nil {
		return nil, err
	}
	result = make([]*v1beta1.ManagedOSVersion, 0, len(objs))
	for _, obj := range objs {
		result = append(result, obj.(*v1beta1.ManagedOSVersion))
	}
	return result, nil
}

type ManagedOSVersionStatusHandler func(obj *v1beta1.ManagedOSVersion, status v1beta1.ManagedOSVersionStatus) (v1beta1.ManagedOSVersionStatus, error)

type ManagedOSVersionGeneratingHandler func(obj *v1beta1.ManagedOSVersion, status v1beta1.ManagedOSVersionStatus) ([]runtime.Object, v1beta1.ManagedOSVersionStatus, error)

func RegisterManagedOSVersionStatusHandler(ctx context.Context, controller ManagedOSVersionController, condition condition.Cond, name string, handler ManagedOSVersionStatusHandler) {
	statusHandler := &managedOSVersionStatusHandler{
		client:    controller,
		condition: condition,
		handler:   handler,
	}
	controller.AddGenericHandler(ctx, name, FromManagedOSVersionHandlerToHandler(statusHandler.sync))
}

func RegisterManagedOSVersionGeneratingHandler(ctx context.Context, controller ManagedOSVersionController, apply apply.Apply,
	condition condition.Cond, name string, handler ManagedOSVersionGeneratingHandler, opts *generic.GeneratingHandlerOptions) {
	statusHandler := &managedOSVersionGeneratingHandler{
		ManagedOSVersionGeneratingHandler: handler,
		apply:                             apply,
		name:                              name,
		gvk:                               controller.GroupVersionKind(),
	}
	if opts != nil {
		statusHandler.opts = *opts
	}
	controller.OnChange(ctx, name, statusHandler.Remove)
	RegisterManagedOSVersionStatusHandler(ctx, controller, condition, name, statusHandler.Handle)
}

type managedOSVersionStatusHandler struct {
	client    ManagedOSVersionClient
	condition condition.Cond
	handler   ManagedOSVersionStatusHandler
}

func (a *managedOSVersionStatusHandler) sync(key string, obj *v1beta1.ManagedOSVersion) (*v1beta1.ManagedOSVersion, error) {
	if obj == nil {
		return obj, nil
	}

	origStatus := obj.Status.DeepCopy()
	obj = obj.DeepCopy()
	newStatus, err := a.handler(obj, obj.Status)
	if err != nil {
		// Revert to old status on error
		newStatus = *origStatus.DeepCopy()
	}

	if a.condition != "" {
		if errors.IsConflict(err) {
			a.condition.SetError(&newStatus, "", nil)
		} else {
			a.condition.SetError(&newStatus, "", err)
		}
	}
	if !equality.Semantic.DeepEqual(origStatus, &newStatus) {
		if a.condition != "" {
			// Since status has changed, update the lastUpdatedTime
			a.condition.LastUpdated(&newStatus, time.Now().UTC().Format(time.RFC3339))
		}

		var newErr error
		obj.Status = newStatus
		newObj, newErr := a.client.UpdateStatus(obj)
		if err == nil {
			err = newErr
		}
		if newErr == nil {
			obj = newObj
		}
	}
	return obj, err
}

type managedOSVersionGeneratingHandler struct {
	ManagedOSVersionGeneratingHandler
	apply apply.Apply
	opts  generic.GeneratingHandlerOptions
	gvk   schema.GroupVersionKind
	name  string
}

func (a *managedOSVersionGeneratingHandler) Remove(key string, obj *v1beta1.ManagedOSVersion) (*v1beta1.ManagedOSVersion, error) {
	if obj != nil {
		return obj, nil
	}

	obj = &v1beta1.ManagedOSVersion{}
	obj.Namespace, obj.Name = kv.RSplit(key, "/")
	obj.SetGroupVersionKind(a.gvk)

	return nil, generic.ConfigureApplyForObject(a.apply, obj, &a.opts).
		WithOwner(obj).
		WithSetID(a.name).
		ApplyObjects()
}

func (a *managedOSVersionGeneratingHandler) Handle(obj *v1beta1.ManagedOSVersion, status v1beta1.ManagedOSVersionStatus) (v1beta1.ManagedOSVersionStatus, error) {
	if !obj.DeletionTimestamp.IsZero() {
		return status, nil
	}

	objs, newStatus, err := a.ManagedOSVersionGeneratingHandler(obj, status)
	if err != nil {
		return newStatus, err
	}

	return newStatus, generic.ConfigureApplyForObject(a.apply, obj, &a.opts).
		WithOwner(obj).
		WithSetID(a.name).
		ApplyObjects(objs...)
}
